"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var babel_parser_1 = require("./babel-parser");
var cacher_1 = require("./utils/cacher");
var resolveExportedComponent_1 = require("./utils/resolveExportedComponent");
// tslint:disable-next-line:no-var-requires
var recast = require("recast");
var ERROR_MISSING_DEFINITION = 'No suitable component definition found';
function parseScript(source, documentation, handlers, options) {
    var plugins = options.lang === 'ts' ? ['typescript'] : ['flow'];
    var ast = cacher_1.default(function () { return recast.parse(source, { parser: babel_parser_1.default({ plugins: plugins }) }); }, source);
    if (!ast) {
        throw new Error(ERROR_MISSING_DEFINITION);
    }
    var componentDefinitions = resolveExportedComponent_1.default(ast);
    if (componentDefinitions.size === 0) {
        throw new Error(ERROR_MISSING_DEFINITION);
    }
    executeHandlers(handlers, componentDefinitions, documentation, ast, options);
}
exports.default = parseScript;
function executeHandlers(localHandlers, componentDefinitions, documentation, ast, opt) {
    return componentDefinitions.forEach(function (compDef, name) {
        if (compDef && name && (!opt.nameFilter || opt.nameFilter.indexOf(name) > -1)) {
            localHandlers.forEach(function (handler) { return handler(documentation, compDef, ast, opt); });
        }
    });
}
//# sourceMappingURL=parse-script.js.map