"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var bt = require("@babel/types");
var getDocblock_1 = require("../utils/getDocblock");
var getDoclets_1 = require("../utils/getDoclets");
// tslint:disable-next-line:no-var-requires
var recast = require("recast");
function eventHandler(documentation, path) {
    recast.visit(path, {
        visitCallExpression: function (pathExpression) {
            if (bt.isMemberExpression(pathExpression.node.callee) &&
                bt.isThisExpression(pathExpression.node.callee.object) &&
                bt.isIdentifier(pathExpression.node.callee.property) &&
                pathExpression.node.callee.property.name === '$emit') {
                var args = pathExpression.node.arguments;
                if (!args.length) {
                    return false;
                }
                var firstArg = args[0];
                if (!bt.isStringLiteral(firstArg)) {
                    return false;
                }
                var eventName = firstArg.value;
                // if this event is documented somewhere else leave it alone
                var evtDescriptor = documentation.getEventDescriptor(eventName);
                // fetch the leading comments on the wrapping expression
                var docblock = getDocblock_1.default(pathExpression.parentPath);
                setEventDescriptor(evtDescriptor, getDoclets_1.default(docblock || ''));
                if (args.length > 1 && !evtDescriptor.type) {
                    evtDescriptor.type = {
                        names: ['undefined'],
                    };
                }
                if (args.length > 2 && !evtDescriptor.properties) {
                    evtDescriptor.properties = [];
                }
                if (evtDescriptor.properties && evtDescriptor.properties.length < args.length - 2) {
                    var i = args.length - 2 - evtDescriptor.properties.length;
                    while (i--) {
                        evtDescriptor.properties.push({
                            type: { names: ['undefined'] },
                            name: "<anonymous" + (args.length - i - 2) + ">",
                        });
                    }
                }
                return false;
            }
            this.traverse(pathExpression);
        },
    });
}
exports.default = eventHandler;
function setEventDescriptor(eventDescriptor, jsDoc) {
    eventDescriptor.description = jsDoc.description || '';
    var nonNullTags = jsDoc.tags ? jsDoc.tags : [];
    var typeTags = nonNullTags.filter(function (tg) { return tg.title === 'type'; });
    eventDescriptor.type = typeTags.length
        ? { names: typeTags.map(function (t) { return t.type.name; }) }
        : undefined;
    var propertyTags = nonNullTags.filter(function (tg) { return tg.title === 'property'; });
    if (propertyTags.length) {
        eventDescriptor.properties = propertyTags.map(function (tg) {
            return { type: { names: [tg.type.name] }, name: tg.name, description: tg.description };
        });
    }
    // remove the property an type tags from the tag array
    var tags = nonNullTags.filter(function (tg) { return tg.title !== 'type' && tg.title !== 'property' && tg.title !== 'event'; });
    if (tags.length) {
        eventDescriptor.tags = tags;
    }
    else {
        delete eventDescriptor.tags;
    }
    return eventDescriptor;
}
exports.setEventDescriptor = setEventDescriptor;
//# sourceMappingURL=eventHandler.js.map