"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var bt = require("@babel/types");
var path = require("path");
var parse_1 = require("../parse");
var adaptExportsToIEV_1 = require("../utils/adaptExportsToIEV");
var makePathResolver_1 = require("../utils/makePathResolver");
var resolveRequired_1 = require("../utils/resolveRequired");
/**
 * Retruns documentation of the component referenced in the extends property of the component
 * @param {NodePath} astPath
 * @param {Array<NodePath>} componentDefinitions
 * @param {string} originalFilePath
 */
function extendsHandler(documentation, componentDefinition, astPath, opt) {
    var extendsVariableName = getExtendsVariableName(componentDefinition);
    // if there is no extends or extends is a direct require
    if (!extendsVariableName) {
        return;
    }
    // get all require / import statements
    var extendsFilePath = resolveRequired_1.default(astPath, [extendsVariableName]);
    var originalDirName = path.dirname(opt.filePath);
    var pathResolver = makePathResolver_1.default(originalDirName, opt.aliases);
    adaptExportsToIEV_1.default(pathResolver, extendsFilePath);
    // only look for documentation in the current project not in node_modules
    if (/^\./.test(extendsFilePath[extendsVariableName].filePath)) {
        var fullFilePath = pathResolver(extendsFilePath[extendsVariableName].filePath);
        parse_1.parseFile(documentation, __assign({}, opt, { filePath: fullFilePath, nameFilter: [extendsFilePath[extendsVariableName].exportName] }));
    }
}
exports.default = extendsHandler;
function getExtendsVariableName(compDef) {
    var extendsVariable = compDef &&
        bt.isClassDeclaration(compDef.node) &&
        compDef.node.superClass &&
        bt.isIdentifier(compDef.node.superClass)
        ? compDef.get('superClass')
        : getExtendsVariableNameFromCompDef(compDef);
    if (extendsVariable) {
        var extendsValue = bt.isProperty(extendsVariable.node)
            ? extendsVariable.node.value
            : extendsVariable.node;
        return extendsValue && bt.isIdentifier(extendsValue) ? extendsValue.name : undefined;
    }
    return undefined;
}
function getExtendsVariableNameFromCompDef(compDef) {
    if (!compDef) {
        return undefined;
    }
    var pathExtends = compDef
        .get('properties')
        .filter(function (p) { return p.node.key.name === 'extends'; });
    return pathExtends.length ? pathExtends[0] : undefined;
}
//# sourceMappingURL=extendsHandler.js.map