"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var bt = require("@babel/types");
var babel_parser_1 = require("../babel-parser");
// tslint:disable-next-line:no-var-requires
var recast = require("recast");
var allowRE = /^(v-bind|:)/;
function propTemplateHandler(documentation, templateAst, options) {
    if (options.functional) {
        propsInAttributes(templateAst, documentation);
        propsInInterpolation(templateAst, documentation);
    }
}
exports.default = propTemplateHandler;
function propsInAttributes(templateAst, documentation) {
    var bindings = templateAst.attrsMap;
    var keys = Object.keys(bindings);
    for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
        var key = keys_1[_i];
        // only look at expressions
        if (allowRE.test(key)) {
            var expression = bindings[key];
            getPropsFromExpression(expression, documentation);
        }
    }
}
function propsInInterpolation(templateAst, documentation) {
    if (templateAst.children) {
        templateAst.children
            .filter(function (c) { return c.type === 2; })
            .forEach(function (expr) {
            getPropsFromExpression(expr.expression, documentation);
        });
    }
}
function getPropsFromExpression(expression, documentation) {
    var ast = babel_parser_1.default({ plugins: ['typescript'] }).parse(expression);
    recast.visit(ast.program, {
        visitMemberExpression: function (path) {
            var obj = path.node ? path.node.object : undefined;
            var propName = path.node ? path.node.property : undefined;
            if (obj &&
                propName &&
                bt.isIdentifier(obj) &&
                obj.name === 'props' &&
                bt.isIdentifier(propName)) {
                var pName = propName.name;
                var p = documentation.getPropDescriptor(pName);
                p.type = { name: 'undefined' };
            }
            return false;
        },
    });
}
//# sourceMappingURL=propHandler.js.map